import re

import requests

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Bitly(SiteVisitor):
    NAME = 'Bitly'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'bit\.ly/(?P<id>\w+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://bit.ly/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://bit.ly/{id}')
        context.create_result('Bitly', url=url, score=1.0)
        context.visit(res.url)


from .linktree import Linktree
from .youtube import Youtube
from .twitter import Twitter
from .bitly import Bitly
from .booth import Booth
from .instagram import Instagram
from .nicovideo import Nicovideo
from .pixiv import Pixiv
from .fanbox import Fanbox
from .twitch import Twitch
from .soundcloud import Soundcloud
from .litlink import LitLink
from .googl import Googl


import re

import requests

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Googl(SiteVisitor):
    NAME = 'Googl'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'goo\.gl/(?P<id>\w+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://goo.gl/{id}')
        context.create_result('Googl', url=url, score=1.0)
        context.visit(res.url)


import random
import re
import string

from .visitor import Result

HTTP_REGEX = f'(https?://)?(www.)?'


def print_result(result: Result, indent_level=0):
    indent = indent_level * "    "
    print(f'{indent}{result.site_name}')
    print(f'{indent}│url  : {result.url}')
    print(f'{indent}│name : {result.title}')
    print(f'{indent}│score: {result.score}')
    print(f'{indent}│links : {result.links}')
    if result.description:
        print(f'{indent}│description: ' + result.description.replace("\n", "\\n"))
    for child in result.children:
        print_result(child, indent_level + 1)


def parse_host(url: str) -> str:
    match = re.search(r'(https?:\/\/)?(www\.)?(?P<host>[\w.]+)/', url)
    if match is None:
        return url
    return match.group('host')

def random_string(length: int) -> str:
    return ''.join(random.choice(string.ascii_letters) for _ in range(length))


import json
import re
from typing import List

import bs4
import requests
from typing_extensions import TypedDict

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Nicovideo(SiteVisitor):
    NAME = 'Nicovideo'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'(sp\.)?nicovideo.jp/(?P<path>user|mylist)/(?P<id>\d+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        if match.group('path') == 'mylist':
            return self.normalize(requests.get(f'https://www.nicovideo.jp/mylist/{match.group("id")}', allow_redirects=True).url)
        return f'nicovideo.jp/user/{match.group("id")}'

    def visit(self, url, context: Context, path: str, id: str):
        url = f'https://www.nicovideo.jp/user/{id}'
        res = requests.get(f'https://www.nicovideo.jp/user/{id}')
        soup = bs4.BeautifulSoup(res.text, 'html.parser')
        element: bs4.Tag = soup.find(attrs={'id': 'js-initial-userpage-data'})  # type: ignore
        if element is None:
            print(f'{id} not found')
            return None

        info: Root = json.loads(element.attrs['data-initial-data'])
        user = info['state']['userDetails']['userDetails']['user']
        context.create_result('Nicovideo', url=url, score=1.0, description=user['description'], profile_picture=user['icons']['large'])

        for link in user['sns']:
            context.visit(link['url'])



class UserLevel(TypedDict):
    currentLevel: int
    nextLevelThresholdExperience: int
    nextLevelExperience: int
    currentLevelExperience: int


class SnsItem0(TypedDict):
    type: str
    label: str
    iconUrl: str
    screenName: str
    url: str


class CoverImage(TypedDict):
    ogpUrl: str
    pcUrl: str
    smartphoneUrl: str


class Icons(TypedDict):
    small: str
    large: str


class User(TypedDict):
    description: str
    decoratedDescriptionHtml: str
    strippedDescription: str
    isPremium: bool
    registeredVersion: str
    followeeCount: int
    followerCount: int
    userLevel: UserLevel
    userChannel: None
    isNicorepoReadable: bool
    sns: List[SnsItem0]
    coverImage: CoverImage
    id: int
    nickname: str
    icons: Icons


class FollowStatus(TypedDict):
    isFollowing: bool


class UserDetails1(TypedDict):
    type: str
    user: User
    followStatus: FollowStatus


class UserDetails(TypedDict):
    userDetails: UserDetails1


class State(TypedDict):
    userDetails: UserDetails


class Root(TypedDict):
    state: State
    nvapi: List



from typing import List, Union
import re
from typing import List

import requests
from typing_extensions import TypedDict
from yt_dlp import YoutubeDL
from urllib import parse

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX

ytdlp = YoutubeDL({
    "quiet": True
})


class Youtube(SiteVisitor):
    NAME = 'Youtube'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'(m\.)?(youtube\.com|youtu\.be)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        info = ytdlp.extract_info(url, download=False, process=False)
        if info is None:
            return url
        if info['extractor'] == 'youtube:tab':
            return info.get('uploader_url') or self.normalize(info['url'])
        return info.get('uploader_url') or info.get('channel_url') # type: ignore

    def visit(self, url, context: Context):
        info = ytdlp.extract_info(url, download=False, process=False)
        assert info
        if 'channel_url' not in info:
            print(f'[Youtube] could not find channel_url for {url}')
            return
        if info['channel_url'] != url:
            info = ytdlp.extract_info(info['channel_url'], download=False, process=False)
            assert info

        context.mark_visited(info['channel_url'])

        profile_picture = None
        if 'thumbnails' in info and len(info['thumbnails']) > 0:
            for thumbnail in info['thumbnails']:
                if 'id' in thumbnail and thumbnail['id'] == 'avatar_uncropped':
                    profile_picture = thumbnail['url']
                    break
            else:
                profile_picture = info['thumbnails'][0]['url']

        context.create_result('Youtube', url=info['channel_url'], score=1.0, name=info['uploader'], description=info['description'], profile_picture=profile_picture)

        res = requests.get(f'{info["channel_url"]}/about')
        match = re.search(r'\"innertubeApiKey\": ?\"(?P<key>\w+)\"', res.text)
        if match is None:
            return

        key = match.group('key')
        about_res = requests.post(f'https://www.youtube.com/youtubei/v1/browse?key={key}&prettyPrint=False', json={
            "context": {
                "client": {
                    "clientName": "WEB",
                    "clientVersion": "2.20201021.01.00",
                    "device": {
                        "deviceCategory": "WEB",
                    },
                    "originalUrl": "https://www.youtube.com/@krr/about"
                }
            },
            'browseId': info['uploader_id']
        })
        about = about_res.json()
        header: Root = about['header']
        if 'headerLinks' not in header['c4TabbedHeaderRenderer']:
            return
        header_links = header['c4TabbedHeaderRenderer']['headerLinks']['channelHeaderLinksRenderer']
        for link in header_links.get('primaryLinks', []) + header_links.get('secondaryLinks', []):
            url = link['navigationEndpoint']['urlEndpoint']['url']
            if url.startswith('https://www.youtube.com/redirect'):
                context.visit(parse.unquote('&q='.join(url.split('&q=')[1:])))
            else:
                context.visit(url)


class WebCommandMetadata(TypedDict):
    url: str
    webPageType: str
    rootVe: int
    apiUrl: str


class CommandMetadata(TypedDict):
    webCommandMetadata: Union[WebCommandMetadata, 'WebCommandMetadata1']


class BrowseEndpoint(TypedDict):
    browseId: str
    canonicalBaseUrl: str


class NavigationEndpoint(TypedDict):
    clickTrackingParams: str
    commandMetadata: CommandMetadata
    browseEndpoint: Union[BrowseEndpoint, 'BrowseEndpoint1']


class ThumbnailsItem0(TypedDict):
    url: str
    width: int
    height: int


class Avatar(TypedDict):
    thumbnails: Union[List['ThumbnailsItem01'], List[ThumbnailsItem0]]


class WebCommandMetadata1(TypedDict):
    url: str
    webPageType: str
    rootVe: int


class UrlEndpoint(TypedDict):
    url: str
    target: str
    nofollow: bool


class NavigationEndpoint1(TypedDict):
    clickTrackingParams: str
    commandMetadata: CommandMetadata
    urlEndpoint: UrlEndpoint


class ThumbnailsItem01(TypedDict):
    url: str


class Title(TypedDict):
    simpleText: str


class PrimaryLinksItem0(TypedDict):
    navigationEndpoint: NavigationEndpoint1
    icon: Avatar
    title: Title


class ChannelHeaderLinksRenderer(TypedDict):
    primaryLinks: List[PrimaryLinksItem0]
    secondaryLinks: List[PrimaryLinksItem0]


class HeaderLinks(TypedDict):
    channelHeaderLinksRenderer: ChannelHeaderLinksRenderer


class AccessibilityData(TypedDict):
    label: str


class Accessibility(TypedDict):
    accessibilityData: AccessibilityData


class SubscriberCountText(TypedDict):
    accessibility: Accessibility
    simpleText: str


class RunsItem0(TypedDict):
    text: str


class ChannelHandleText(TypedDict):
    runs: List[RunsItem0]


class BrowseEndpoint1(TypedDict):
    browseId: str
    params: str
    canonicalBaseUrl: str


class MoreIcon(TypedDict):
    iconType: str


class ChannelTaglineRenderer(TypedDict):
    content: str
    maxLines: int
    moreLabel: str
    moreEndpoint: NavigationEndpoint
    moreIcon: MoreIcon


class Tagline(TypedDict):
    channelTaglineRenderer: ChannelTaglineRenderer


class C4TabbedHeaderRenderer(TypedDict):
    channelId: str
    title: str
    navigationEndpoint: NavigationEndpoint
    avatar: Avatar
    banner: Avatar
    headerLinks: HeaderLinks
    subscriberCountText: SubscriberCountText
    tvBanner: Avatar
    mobileBanner: Avatar
    trackingParams: str
    channelHandleText: ChannelHandleText
    style: str
    videosCountText: ChannelHandleText
    tagline: Tagline


class Root(TypedDict):
    c4TabbedHeaderRenderer: C4TabbedHeaderRenderer



import re
from typing import List

import requests
from typing_extensions import TypedDict

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Fanbox(SiteVisitor):
    NAME = 'Fanbox'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'(?P<id>\w+).fanbox.cc', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://{match.group("id")}.fanbox.cc'

    def visit(self, url, context: Context, id: str):
        url = f'https://{id}.fanbox.cc'
        session = requests.Session()
        res = session.get(f'https://{id}.fanbox.cc')

        creator_res = session.get(f'https://api.fanbox.cc/creator.get?creatorId={id}', headers={
            'accept': 'application/json',
            'origin': f'https://{id}.fanbox.cc',
            'referer': f'https://{id}.fanbox.cc/'
        })

        info: Root = creator_res.json()
        context.create_result('Fanbox', url=url, name=info['body']['user']['name'], score=1.0, description=info['body']['description'], profile_picture=info['body']['user']['iconUrl'])

        for link in info['body']['profileLinks']:
            context.visit(link)


class User(TypedDict):
    userId: str
    name: str
    iconUrl: str


class ProfileItemsItem0(TypedDict):
    id: str
    type: str
    imageUrl: str
    thumbnailUrl: str


class Body(TypedDict):
    user: User
    creatorId: str
    description: str
    hasAdultContent: bool
    coverImageUrl: str
    profileLinks: List[str]
    profileItems: List[ProfileItemsItem0]
    isFollowed: bool
    isSupported: bool
    isStopped: bool
    isAcceptingRequest: bool
    hasBoothShop: bool


class Root(TypedDict):
    body: Body



import re
from dataclasses import dataclass, field
from typing import List, Optional, Protocol

HTTP_REGEX = f'(https?://)?(www.)?'


@dataclass
class Result:
    site_name: Optional[str]
    url: str
    score: float
    title: Optional[str]
    description: Optional[str]
    profile_picture: Optional[str]

    children: List['Result'] = field(default_factory=list)
    links: List['str'] = field(default_factory=list)


class Visitor(Protocol):
    def visit(self, url: str, context: 'Context', **kwargs) -> Result: ...

    def mark_visited(self, url: str) -> None: ...


class Context:
    def __init__(self, url: str, visitor: Visitor, parent: Optional['Context'] = None):
        self.url = url
        self.visitor = visitor
        self.parent = parent
        self.result: Optional[Result] = None

    def create_result(self, site_name: str, url: str, score: float = 1.0, name: Optional[str] = None, description: Optional[str] = None, profile_picture: Optional[str] = None) -> Result:
        self.result = Result(site_name=site_name, url=url, score=score, title=name, description=description, profile_picture=profile_picture)

        if self.parent and self.parent.result:
            self.parent.result.children.append(self.result)

        return self.result

    def link(self, url: str) -> None:
        if self.result is not None:
            self.result.links.append(url)

    def mark_visited(self, url: str) -> None:
        self.visitor.mark_visited(url)

    def create(self, url: str) -> 'Context':
        return Context(url=url, visitor=self.visitor, parent=self)

    def visit(self, url: str) -> None:
        self.visitor.visit(url, self.create(url))

    def __repr__(self) -> str:
        return f'Context(url={self.url!r})'


class SiteVisitor:
    NAME: Optional[str] = None
    URL_REGEX: Optional[re.Pattern] = None

    def match(self, url, context: Context) -> Optional[re.Match]:
        if self.URL_REGEX is None:
            raise NotImplementedError()
        return self.URL_REGEX.match(url)

    def normalize(self, url: str) -> str:
        return url

    def visit(self, url, context: Context, **kwargs) -> Optional[Result]:
        raise NotImplementedError()



from typing_extensions import TypedDict
from typing import List
import re

import requests

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Instagram(SiteVisitor):
    NAME = 'Instagram'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'instagram\.com/(?P<id>\w+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://www.instagram.com/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        session = requests.Session()
        session.headers = {
            'authority': 'www.instagram.com',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'referer': f'https://www.instagram.com/{id}/',
            'sec-ch-prefers-color-scheme': 'dark',
            'sec-ch-ua': '"Not?A_Brand";v="8", "Chromium";v="108", "Microsoft Edge";v="108"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.150 Safari/537.36',
            'x-asbd-id': '198387',
            'x-ig-www-claim': '0',
            'x-requested-with': 'XMLHttpRequest'
        }

        url = f'https://www.instagram.com/{id}/'
        res = requests.get(url)
        match = re.search(r'\"X-IG-App-ID\": ?\"(?P<id>\d{15})\"', res.text)
        if match is None:
            print(f'No X-IG-App-ID found in {url}')
            return
        session.headers['x-ig-app-id'] = match.group('id')
        
        csrf_res = requests.get('https://www.instagram.com/ajax/bz?__d=dis')
        session.headers['x-csrftoken'] = csrf_res.cookies.get_dict()['csrftoken']

        info_res = requests.get(f'https://www.instagram.com/api/v1/users/web_profile_info/?username={id}')
        if not info_res.ok or info_res.history:
            print('[Instagram] Blocked by Instagram')
            context.create_result('Instagram', url=url, name=id, score=0.0, description='Blocked by Instagram')
            return
        info: Root = info_res.json()
        user = info['data']['user']
        context.create_result('Instagram', url=url, score=1.0, description=user['biography'])

        for link in user['bio_links']:
            context.visit(link['url'])


class BioLinksItem0(TypedDict):
    title: str
    lynx_url: str
    url: str
    link_type: str


class BiographyWithEntities(TypedDict):
    raw_text: str
    entities: List


class EdgeFollowedBy(TypedDict):
    count: int


class EdgeMutualFollowedBy(TypedDict):
    count: int
    edges: List


class User(TypedDict):
    biography: str
    bio_links: List[BioLinksItem0]
    biography_with_entities: BiographyWithEntities
    blocked_by_viewer: bool
    restricted_by_viewer: None
    country_block: bool
    external_url: str
    external_url_linkshimmed: str
    edge_followed_by: EdgeFollowedBy
    fbid: str
    followed_by_viewer: bool
    edge_follow: EdgeFollowedBy
    follows_viewer: bool
    full_name: str
    group_metadata: None
    has_ar_effects: bool
    has_clips: bool
    has_guides: bool
    has_channel: bool
    has_blocked_viewer: bool
    highlight_reel_count: int
    has_requested_viewer: bool
    hide_like_and_view_counts: bool
    id: str
    is_business_account: bool
    is_professional_account: bool
    is_supervision_enabled: bool
    is_guardian_of_viewer: bool
    is_supervised_by_viewer: bool
    is_supervised_user: bool
    is_embeds_disabled: bool
    is_joined_recently: bool
    guardian_id: None
    business_address_json: None
    business_contact_method: str
    business_email: None
    business_phone_number: None
    business_category_name: None
    overall_category_name: None
    category_enum: None
    category_name: str
    is_private: bool
    is_verified: bool
    edge_mutual_followed_by: EdgeMutualFollowedBy
    profile_pic_url: str
    profile_pic_url_hd: str
    requested_by_viewer: bool
    should_show_category: bool
    should_show_public_contacts: bool
    transparency_label: None
    transparency_product: str
    username: str
    connected_fb_page: None
    pronouns: List


class Data(TypedDict):
    user: User


class Root(TypedDict):
    data: Data
    status: str



import re

import requests

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Twitch(SiteVisitor):
    NAME = 'Twitch'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'twitch.tv/(?P<id>\w+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://www.twitch.tv/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        url = f'https://www.twitch.tv/{id}'

        res = requests.get(url)
        match = re.search(r'clientId ?= ?"(?P<token>kimne78kx3ncx6brgo4mv6wki5h1ko)"', res.text)
        if match is None:
            print(f'[Twitch] Could not find token for {url}')
            return
        token = match.group('token')

        res = requests.post('https://gql.twitch.tv/gql', json=[
            {
                "operationName": "ChannelRoot_AboutPanel",
                "variables": {
                    "channelLogin": id,
                    "skipSchedule": True
                },
                "extensions": {
                    "persistedQuery": {
                        "version": 1,
                        "sha256Hash": "6089531acef6c09ece01b440c41978f4c8dc60cb4fa0124c9a9d3f896709b6c6"
                    }
                }
            }
        ], headers={
            'Client-Id': token,
        })

        info: Root = res.json()
        info

        info: Root = res.json()
        user = info[0]["data"]["user"]
        context.create_result('Twitch', url=url, name=user["displayName"], score=1.0, description=user["description"], profile_picture=user['profileImageURL'])

        for link in user['channel']["socialMedias"]:
            context.visit(link["url"])

from typing import List

from typing_extensions import TypedDict


class Followers(TypedDict):
    totalCount: int
    __typename: str


class SocialMediasItem0(TypedDict):
    id: str
    name: str
    title: str
    url: str
    __typename: str


class Channel(TypedDict):
    id: str
    socialMedias: List[SocialMediasItem0]
    __typename: str


class LastBroadcast(TypedDict):
    id: str
    game: None
    __typename: str


class Videos(TypedDict):
    edges: List
    __typename: str


class User(TypedDict):
    id: str
    description: str
    displayName: str
    isPartner: bool
    primaryColorHex: str
    profileImageURL: str
    followers: Followers
    channel: Channel
    lastBroadcast: LastBroadcast
    primaryTeam: None
    videos: Videos
    __typename: str


class Data(TypedDict):
    currentUser: None
    user: User


class Extensions(TypedDict):
    durationMilliseconds: int
    operationName: str
    requestID: str


class RootItem0(TypedDict):
    data: Data
    extensions: Extensions


Root = List[RootItem0]


import json
import re
from typing import Dict, List, Optional, TypeAlias

import bs4
import requests
from typing_extensions import TypedDict

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Pixiv(SiteVisitor):
    NAME = 'Pixiv'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'pixiv.net/users/(?P<id>\d+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://pixiv.net/users/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://pixiv.net/users/{id}')
        soup = bs4.BeautifulSoup(res.text, 'html.parser')
        meta_element: bs4.Tag = soup.find('meta', attrs={'name': 'preload-data', 'id': 'meta-preload-data'})  # type: ignore
        info: Root = json.loads(meta_element.attrs['content'])

        if len(info['user'].values()) > 1:
            print(f'[Pixiv] User is must be unique: {id}')
            return

        for user in info['user'].values():
            context.create_result('Pixiv', url=url, name=user['name'], score=1.0, description=user['comment'], profile_picture=user['imageBig'])
            if 'webpage' in user and user['webpage'] is not None:
                context.visit(user['webpage'])
            if not user['social']:
                return
            for link in user['social'].values():
                context.visit(link['url'])


class Background(TypedDict):
    repeat: None
    color: None
    url: str
    isPrivate: bool


class Social(TypedDict):
    url: str


class Region(TypedDict):
    name: str
    region: str
    prefecture: str
    privacyLevel: str


class Age(TypedDict):
    name: Optional[str]
    privacyLevel: Optional[str]


class UserDetail(TypedDict):
    userId: str
    name: str
    image: str
    imageBig: str
    premium: bool
    isFollowed: bool
    isMypixiv: bool
    isBlocking: bool
    background: Background
    sketchLiveId: None
    partial: int
    acceptRequest: bool
    sketchLives: List
    following: int
    mypixivCount: int
    followedBack: bool
    comment: str
    commentHtml: str
    webpage: str
    social: Dict[str, Social]
    canSendMessage: bool
    region: Region
    age: Age
    birthDay: Age
    gender: Age
    job: Age
    workspace: None
    official: bool
    group: None


User: TypeAlias = Dict[str, UserDetail]


class Root(TypedDict):
    timestamp: str
    user: User



import json
import re
from typing import List, Union

import bs4
import requests
from typing_extensions import TypedDict

from ..helper import HTTP_REGEX
from ..visitor import Context, SiteVisitor


class LitLink(SiteVisitor):
    NAME = 'LitLink'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'lit\.link/(?P<id>\w+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://lit.link/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://lit.link/{id}')
        soup = bs4.BeautifulSoup(res.text, 'html.parser')
        data_element = soup.find(attrs={'id': '__NEXT_DATA__'})
        if data_element is None:
            print(f'[LitLink] Could not find data element for {url}')
            return
        data: Root = json.loads(data_element.get_text())
        profile: ProfileData = json.loads(data['props']['pageProps']['profileString'])

        context.create_result('LitLink', url=url, name=profile['name'], score=1.0, description=profile['profileText'], profile_picture=profile['pictureUrl'])

        for link in profile['snsIconLink']['details']:
            if 'url' not in link:
                continue
            context.visit(link['url'])


class Birthday(TypedDict):
    seconds: int
    nanoseconds: int


class SnsIconLink(TypedDict):
    details: Union[List, List['DetailsItem0']]


class ButtonLink(TypedDict):
    iconUrl: str
    title: str
    description: str
    url: str
    urlType: str


class DetailsItem0(TypedDict):
    linkType: str
    url: str


class CreatorDetailLayout(TypedDict):
    fontFamily: str
    fontColor: str
    fontSize: str
    textAlign: str
    backgroundImageUrl: str
    backgroundColor: str
    backgroundGradation: str
    backgroundOverlayColor: str
    linkShapeType: str
    linkShapeColor: str
    template: str


class CategorySettingsItem0(TypedDict):
    id: str
    name: str
    english: str


class GenreSettingsItem0(TypedDict):
    id: str
    name: str
    english: str
    category_settings: List[CategorySettingsItem0]
    categorySettings: List[CategorySettingsItem0]


class SnsActivitySetting(TypedDict):
    genreSettings: List[GenreSettingsItem0]


class ProfileData(TypedDict):
    uid: str
    name: str
    sex: str
    birthday: Birthday
    genre: str
    profileText: str
    url: str
    pictureUrl: str
    pictureType: str
    snsIconLink: SnsIconLink
    profileLink: SnsIconLink
    creatorDetailLayout: CreatorDetailLayout
    snsActivitySetting: SnsActivitySetting


class Account(TypedDict):
    email: str
    url: str
    emailUpdateResponse: None
    isLoading: bool
    emailConnected: bool
    urlUpdateResponse: None


class FontColor(TypedDict):
    r: int
    g: int
    b: int
    a: int


class CreatorDetailEdit(TypedDict):
    isEdit: bool
    editingProfile: None
    editingSnsIconLinkDetails: List
    editingProfileLinkDetails: List
    editingCreatorDetailLayout: None
    selectedBackgroundCategory: str
    fontColor: FontColor
    backgroundColor: FontColor
    backgroundGradationStartColor: FontColor
    backgroundGradationEndColor: FontColor
    backgroundGradationColorPaletteIndex: int
    isActiveUrlPastingOnText: bool
    linkShapeColor: FontColor
    profileLinkWidth: int
    isLoading: bool
    snsActivityGenres: List
    imageUpLoading: bool
    showSavedToast: bool
    toastText: str
    profileLinkUrlType: None
    profileLinkErrors: List
    modalSnsType: None
    snsModalDefaultUrl: str
    multipleImageLinkIndex: int
    fourImageLinkIndex: int
    selectedIndexOnImageOrSnsModal: int
    isCheckedOpenCategory: bool
    currentOpenedGenreIndex: int
    showIconQrCode: bool
    hasSavedProfile: bool
    backgroundImageUrlForOverlay: str


class GenreCategory(TypedDict):
    isLoading: bool
    selectedMoreThanOne: bool
    genreCategoryList: List
    openedGenreCategoryIds: List


class Profile(TypedDict):
    isLoggedIn: bool
    showCopiedMessage: bool
    showIconQrCode: bool
    profile: None


class LineLogin(TypedDict):
    lineLoginResponse: None
    isLoading: bool


class Login(TypedDict):
    loginResponse: None
    isLoading: bool
    loginErrorMessageId: None


class ConfirmationModalOptions(TypedDict):
    modalText: str
    positiveText: str
    negativeText: str


class SelectBackgroundImageModalOptions(TypedDict):
    isButtonLinkDesignImage: bool


class SelectImageModalOptions(TypedDict):
    isMultipleImageLink: bool
    isButtonLink: bool


class Modal(TypedDict):
    modalOpened: bool
    modalComponentName: str
    masterModalId: str
    confirmationModalOptions: ConfirmationModalOptions
    selectBackgroundImageModalOptions: SelectBackgroundImageModalOptions
    selectImageModalOptions: SelectImageModalOptions


class LineMessaging(TypedDict):
    lineMessaging: None
    isLoading: bool


class PasswordReminder(TypedDict):
    passwordReminderResponse: None
    isLoading: bool
    isCompletedSendEmail: bool
    hasErrorResponse: bool


class PasswordChange(TypedDict):
    passwordChangeResponse: None
    isLoading: bool


class SignUp(TypedDict):
    singUpAuthResponse: None
    signUpByLineResponse: None
    isLoading: bool
    registeredAlready: bool
    hasAccountByEmailAuth: bool
    defaultEmail: str
    hasErrorSignupResponse: bool


class FirebaseAuth(TypedDict):
    firebaseUser: None
    isAuthLoading: bool
    isResendEmailVerificationSucceeded: None


class SignupDetail(TypedDict):
    isInstagramConnected: bool
    isTwitterConnected: bool
    isLoading: bool
    isVerifiedUrl: None


class LineSignup(TypedDict):
    lineSignUpResponse: None
    isLoading: bool


class DatasetsItem0(TypedDict):
    label: str
    backgroundColor: str
    borderColor: str
    data: List


class UserGraphAccessLog(TypedDict):
    labels: List
    datasets: List[DatasetsItem0]


class Analytics(TypedDict):
    displayPeriod: str
    urlSortType: str
    topSortType: str
    isUrlSortAscendant: bool
    isTopSortAscendant: bool
    isReferralSortAscendant: bool
    isDeviceSortAscendant: bool
    pvCounts: int
    clickCounts: int
    accessTopTableSortType: str
    userTodayAccessLog: None
    userOneWeekAccessLog: None
    userOneMonthAccessLog: None
    userThreeMonthsAccessLog: None
    userSixMonthsAccessLog: None
    userOneYearAccessLog: None
    userAllAccessLog: None
    userGraphAccessLog: UserGraphAccessLog
    userUrlAccessLogs: List
    userTopAccessLogs: List
    userReferralAccessLogs: List
    userDeviceAccessLogs: List
    isAnalyticsStateLoading: bool
    urlAddedAreaHeight: int
    topAddedAreaHeight: int
    referralAddedAreaHeight: int
    isShowingMoreOnUrl: bool
    isShowingMoreOnTop: bool
    isShowingMoreOnReferral: bool
    isAnalyticsApiError: bool
    isShowingToast: bool
    apiError: None


class Notification(TypedDict):
    isLoading: bool
    selectedNotification: None


class CreatorDetailEditTutorial(TypedDict):
    editingCreatorPreferance: None
    tutorialCount: int
    isTutorialButtonEditDone: bool
    isTutorialLinkDraggerDone: bool
    isTutorialLinkEditDone: bool


class AccountDelete(TypedDict):
    isLoading: bool
    isSucceededDelete: None


class ProfileImageNftModal(TypedDict):
    pass


class SignupGenre(TypedDict):
    isSucceeded: None
    isLoading: bool


class InitialState(TypedDict):
    account: Account
    creatorDetailEdit: CreatorDetailEdit
    genreCategory: GenreCategory
    profile: Profile
    lineLogin: LineLogin
    login: Login
    modal: Modal
    lineMessaging: LineMessaging
    passwordReminder: PasswordReminder
    passwordChange: PasswordChange
    signUp: SignUp
    firebaseAuth: FirebaseAuth
    signupDetail: SignupDetail
    lineSignup: LineSignup
    analytics: Analytics
    notification: Notification
    creatorDetailEditTutorial: CreatorDetailEditTutorial
    accountDelete: AccountDelete
    profileImageNFTModal: ProfileImageNftModal
    signupGenre: SignupGenre


class PageProps(TypedDict):
    initialState: InitialState
    profileString: str
    ogpImageUrl: str


class Props(TypedDict):
    pageProps: PageProps
    __N_SSP: bool


class Query(TypedDict):
    creatorUrl: str


class Root(TypedDict):
    props: Props
    page: str
    query: Query
    buildId: str
    isFallback: bool
    gssp: bool
    locale: str
    locales: List[str]
    defaultLocale: str
    scriptLoader: List



import argparse
from .visitor import Result
from .iwashi import visit
from .helper import print_result


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--server', action='store_true')
    parser.add_argument('-url', type=str, required=False)
    args = parser.parse_args()
    if args.server:
        from .server import server
        server.run_server()
    else:
        result = visit(args.url)
        assert result
        print('\n' * 4)
        print_result(result)


if __name__ == "__main__":
    main()



import re
from typing import Optional, Dict

import requests

from ..helper import HTTP_REGEX
from ..visitor import Context, SiteVisitor


class Misskey(SiteVisitor):
    NAME = 'Misskey'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'nicovideo.jp/user/(?P<id>\d+)', re.IGNORECASE)
    
    def __init__(self) -> None:
        self._cache: Dict[str, bool] = {}

    def match(self, url, context: Context) -> Optional[re.Match]:
        match = re.search(HTTP_REGEX + r'(?P<host>[\w.]+)', url)
        if match is None:
            return None

        host = match.group('host')
        return


    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://bit.ly/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://bit.ly/{id}', stream=True)
        context.visit(res.url)



import re

import bs4
import requests

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Booth(SiteVisitor):
    NAME = 'Booth'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'(?P<id>\w+).booth.pm', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://{match.group("id")}.booth.pm'

    def visit(self, url, context: Context, id: str):
        res = requests.get(f'https://{id}.booth.pm')
        soup = bs4.BeautifulSoup(res.text, 'html.parser')
        name_element: bs4.Tag = soup.find(attrs={'class': 'home-link-container__nickname'}) # type: ignore
        name = name_element is not None and name_element.find('a') or None
        desc_element: bs4.Tag = soup.find(attrs={'class': 'description'}) # type: ignore
        description = desc_element is not None and desc_element.find(attrs={'v-html': 'formattedDescription'}) or None
        avater_element: bs4.Tag = soup.find(attrs={'class': 'avatar-image'}) # type: ignore

        context.create_result(
            'Booth',
            url=url,
            name=name.text if name is not None else None,
            score=1.0,
            description=description.text if description is not None else None,
            profile_picture=avater_element.attrs['style'].split('url(')[1].split(')')[0] if avater_element is not None else None
        )

        for link in desc_element.find_all(attrs={'class': 'shop-contacts__link'}):
            link: bs4.Tag = link.find('a') # type: ignore
            if link.has_attr('href'):
                context.visit(link.attrs['href'])


import json
import re
from typing import List

import requests
from typing_extensions import TypedDict

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Soundcloud(SiteVisitor):
    NAME = 'Soundcloud'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'soundcloud\.com/(?P<id>[-\w]+)', re.IGNORECASE)

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.match(url)
        if match is None:
            return url
        return f'https://soundcloud.com/{match.group("id")}'

    def visit(self, url, context: Context, id: str):
        url = f'https://soundcloud.com/{id}'
        res = requests.get(url)
        info_json = re.search(r'window\.__sc_hydration ?= ?(?P<info>.+);', res.text)
        if info_json is None:
            print(f'[Soundcloud] Could not find info for {url}')
            return
        info_list: Root = json.loads(info_json.group('info'))
        for info in info_list:
            if info['hydratable'] == 'user':
                break
        else:
            print(f'[Soundcloud] Could not find user info for {url}')
            return

        context.create_result('Soundcloud', url=url, name=info['data']['username'], score=1.0, description=info['data']['description'], profile_picture=info['data']['avatar_url'])
        pass
        client_id_res = requests.get("https://a-v2.sndcdn.com/assets/0-bf97f26a.js")
        match = re.search(r"client_id: ?\"(?P<client_id>\w{32})\"", client_id_res.text)
        if match is None:
            print(f'[Soundcloud] Could not find client_id for {url}')
            return
        client_id = match.group('client_id')

        "https://api-v2.soundcloud.com/users/soundcloud:users:104832223/web-profiles?client_id=SDvic69dtCia3c4tYqKIhC6j7UfTPHLC&app_version=1678362857&app_locale=en"
        profile_res = requests.get(f'https://api-v2.soundcloud.com/users/{info["data"]["urn"]}/web-profiles?client_id={client_id}&app_version=1678362857&app_locale=en', headers={
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.69'
        })
        profile: List[ProfileItem] = profile_res.json()
        for item in profile:
            context.visit(item['url'])


class ProfileItem(TypedDict):
    url: str
    network: str
    title: str


class Product(TypedDict):
    id: str


class CreatorSubscriptionsItem0(TypedDict):
    product: Product


class VisualsItem0(TypedDict):
    urn: str
    entry_time: int
    visual_url: str


class Visuals(TypedDict):
    urn: str
    enabled: bool
    visuals: List[VisualsItem0]
    tracking: None


class Badges(TypedDict):
    pro: bool
    pro_unlimited: bool
    verified: bool


class Data4(TypedDict):
    avatar_url: str
    city: str
    comments_count: int
    country_code: str
    created_at: str
    creator_subscriptions: List[CreatorSubscriptionsItem0]
    creator_subscription: CreatorSubscriptionsItem0
    description: str
    followers_count: int
    followings_count: int
    first_name: str
    full_name: str
    groups_count: int
    id: int
    kind: str
    last_modified: str
    last_name: str
    likes_count: int
    playlist_likes_count: int
    permalink: str
    permalink_url: str
    playlist_count: int
    reposts_count: None
    track_count: int
    uri: str
    urn: str
    username: str
    verified: bool
    visuals: Visuals
    badges: Badges
    station_urn: str
    station_permalink: str
    url: str


class RootItem0(TypedDict):
    hydratable: str
    data: Data4


Root = List[RootItem0]



from setuptools import setup, find_packages

with open('README.rst', mode='r', encoding='utf-8') as f:
    long_description = f.read()

author = 'am230'
name = 'iwashi'
py_modules = [name]

setup(
    name=name,
    version="1.0.0",
    keywords=("generator"),
    description="Link Tree Generator",
    long_description=long_description,
    install_requires=["Flask", "pytohtml", "py2js", "bs4", "yt-dlp"],
    license="MIT Licence",
    long_description_content_type='text/x-rst',
    packages=find_packages(),
    url=f"https://github.com/{author}/{name}",
    author=author,
    author_email="am.230@outlook.jp",
    py_modules=py_modules,
    platforms="any",
)



import functools
import json
import re
from typing import List, Optional, Union

import bs4
import requests
from typing_extensions import TypedDict

from ..visitor import Context, SiteVisitor
from ..helper import HTTP_REGEX


class Twitter(SiteVisitor):
    NAME = 'Twitter'
    URL_REGEX: re.Pattern = re.compile(HTTP_REGEX + r'twitter\.com/@?(?P<id>\w+)', re.IGNORECASE)

    def __init__(self) -> None:
        pass

    def normalize(self, url: str) -> str:
        match = self.URL_REGEX.search(url)
        if match is None:
            return url
        return f'https://twitter.com/{match.group("id")}'

    @functools.cached_property
    def authorization(self) -> str:
        res = requests.get('https://abs.twimg.com/responsive-web/client-web/main.28fc48ca.js')
        match = re.search("(AAAAA.*?)\"", res.text)
        assert match
        return "Bearer " + match.group(1)

    def visit(self, url, context: Context, id: str):
        headers = {
            'accept': '*/*',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'ja',
            'authorization': self.authorization,
        }

        headers['x-guest-token'] = requests.post('https://api.twitter.com/1.1/guest/activate.json', headers=headers).json()['guest_token']

        res = requests.get(
            'https://api.twitter.com/graphql/rePnxwe9LZ51nQ7Sn_xN_A/UserByScreenName',
            params={
                'variables': json.dumps({
                    "screen_name": id,
                    "withSafetyModeUserFields": True,
                    "withSuperFollowsUserFields": True
                }),
                'features': json.dumps({
                    "responsive_web_twitter_blue_verified_badge_is_enabled": True,
                    "responsive_web_graphql_exclude_directive_enabled": True,
                    "verified_phone_label_enabled": False,
                    "responsive_web_graphql_skip_user_profile_image_extensions_enabled": False,
                    "responsive_web_graphql_timeline_navigation_enabled": True
                })
            },
            headers=headers
        )
        info: Root = json.loads(res.text)
        
        result = info['data']['user']['result']
        if result['__typename'] == 'UserUnavailable':
            context.create_result('Twitter', url=url, name='<UserUnavailable>', score=1.0)
            return
            
        data = result['legacy']
        context.create_result('Twitter', url=url, name=data['name'], score=1.0, description=data['description'], profile_picture=data['profile_image_url_https'])

        if 'url' not in data['entities']:
            return

        for link in data['entities']['url']['urls']:
            context.visit(link['expanded_url'])

        for link in data['entities']['description']['urls']:
            context.visit(link['expanded_url'])


class LocationsItem0(TypedDict):
    line: int
    column: int


class Tracing(TypedDict):
    trace_id: str


class Extensions(TypedDict):
    name: str
    source: str
    code: int
    kind: str
    tracing: Tracing


class ErrorsItem0(TypedDict):
    message: str
    locations: List[LocationsItem0]
    path: List[str]
    extensions: Extensions
    code: int
    kind: str
    name: str
    source: str
    tracing: Tracing


class AffiliatesHighlightedLabel(TypedDict):
    pass


class UrlsItem0(TypedDict):
    display_url: str
    expanded_url: str
    url: str
    indices: List[int]


class Description(TypedDict):
    urls: List[UrlsItem0]


class Entities(TypedDict):
    description: Description
    url: Description


class Rgb(TypedDict):
    blue: int
    green: int
    red: int


class PaletteItem0(TypedDict):
    percentage: float
    rgb: Rgb


class Ok(TypedDict):
    palette: List[PaletteItem0]


class R(TypedDict):
    ok: Ok


class MediaColor(TypedDict):
    r: R


class ProfileBannerExtensions(TypedDict):
    mediaColor: MediaColor


class Legacy(TypedDict):
    created_at: str
    default_profile: bool
    default_profile_image: bool
    description: str
    entities: Entities
    fast_followers_count: int
    favourites_count: int
    followers_count: int
    friends_count: int
    has_custom_timelines: bool
    is_translator: bool
    listed_count: int
    location: str
    media_count: int
    name: str
    normal_followers_count: int
    pinned_tweet_ids_str: List[str]
    possibly_sensitive: bool
    profile_banner_extensions: ProfileBannerExtensions
    profile_banner_url: str
    profile_image_extensions: ProfileBannerExtensions
    profile_image_url_https: str
    profile_interstitial_type: str
    screen_name: str
    statuses_count: int
    translator_type: str
    url: str
    verified: bool
    withheld_in_countries: List


class Result(TypedDict):
    __typename: str
    id: str
    rest_id: str
    affiliates_highlighted_label: AffiliatesHighlightedLabel
    is_blue_verified: bool
    legacy: Legacy
    business_account: AffiliatesHighlightedLabel
    legacy_extended_profile: AffiliatesHighlightedLabel
    is_profile_translatable: bool
    verification_info: AffiliatesHighlightedLabel


class User(TypedDict):
    result: Result


class Data(TypedDict):
    user: User


class Root(TypedDict):
    errors: List[ErrorsItem0]
    data: Data



from typing import List, MutableSet, Optional
from .visitor import Visitor, SiteVisitor, Context, Result
from .helper import parse_host

class Iwashi(Visitor):
    def __init__(self) -> None:
        self.visitors: List[SiteVisitor] = []
        self.visited: MutableSet[str] = set()

    def add_visitor(self, visitor: SiteVisitor) -> None:
        self.visitors.append(visitor)

    def is_visited(self, url: str) -> bool:
        return url in self.visited

    def mark_visited(self, url: str) -> bool:
        url = url.lower()
        if self.is_visited(url):
            return False
        self.visited.add(url)
        return True

    def visit(self, url: str, context: Optional[Context] = None) -> Optional[Result]:
        context = context or Context(url=url, visitor=self)
        for visitor in self.visitors:
            match = visitor.match(url, context)
            if match is None:
                continue

            normalized = visitor.normalize(url)
            if self.mark_visited(normalized):
                match = visitor.match(normalized, context)
                if match is not None:
                    visitor.visit(normalized, context, **match.groupdict())
            elif context.parent is not None:
                context.parent.link(normalized)
            break
        else:
            context.create_result(site_name=parse_host(url), url=url, score=1.0)
            self.mark_visited(url)
            print('[Iwashi]', '[No Visitor Found]', url)

        return context.result

def visit(url: str, iwashi: Optional[Iwashi] = None) -> Optional[Result]:
    if iwashi is None:
        iwashi = Iwashi()
        from . import visitors
        for attr in dir(visitors):
            value = getattr(visitors, attr)
            if attr.startswith('_'):
                continue
            if isinstance(value, type) and issubclass(value, SiteVisitor):
                iwashi.add_visitor(value())

    return iwashi.visit(url)